package com.xmatters.datagrid;

import org.infinispan.Cache;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.util.concurrent.NotifyingFuture;

public class CacheTest {
	private static final int MAX_ITEMS = 100000;
	private static final String LONG_MSG = ": Starting with version 4.1, the Infinispan distribution contains a server module that implements "
			+ "the Memcached text protocol. This allows Memcached clients to talk to one or serveral Infinispan backed Memcached "
			+ "servers. These servers could either be working standalone just like Memcached does where each server acts independently "
			+ "and does not communicate with the rest, or they could be clustered where servers replicate or distribute their contents to "
			+ "other Infinispan backed Memcached servers, thus providing clients with failover capabilities.Infinispan does not attempt to "
			+ "evenly split the hash space between nodes  by not trying to split it evenly, it means that if a node joins or leaves the grid, "
			+ "there is no need to adjust the ownership of every node, just the neighbours of the joiner/leaver. "
			+ "This has a positive impact on network traffic. However this can mean that some nodes take on substantially "
			+ "larger portions of the hash space than others. This, combined with potential irregularities in the hash functions "
			+ "of keys, can mean the distribution of entries across the grid becomes poor. In order to address the irregularities "
			+ "in the hash of keys, Infinispan uses an advanced hashing function (Murmur Hash 3) by default, as well as using a bit "
			+ "spreader. In order to address the irregularities in the node distribution, Infinispan uses virtual nodes."
			+ "First, let's consider how virtual nodes help conceptually by taking a couple of distribution examples, and "
			+ "armed with that knowledge, look at how Infinispan uses them."
			+ "Consider a hash space of 1000 (there are 1000 buckets into which data can be placed). If there were "
			+ "two nodes, it is possible you can have 1 node being used for 1 bucket, and 1 node for 999 buckets (this "
			+ "is the most pessimistic distribution!). If there were 200 nodes, the worst distribution of node would "
			+ "end up being 199 nodes responsible for one bucket each (199 buckets in total), and 1 node being"
			+ " responsible for 801 buckets. If there were 1000 nodes, then each node must be responsible for 1 bucket "
			+ "each. From this we can deduce that as the number of nodes tends to the size of the hash space, that the "
			+ "distribution of buckets to nodes improves.A guiding principle of Infinispan is that it always uses an "
			+ "algorithm to locate a key in the hash space, never allowing the node on which the entry is stored to "
			+ "be specified manually. This scheme allows any node to know which nodes owns a key, without having to "
			+ "distribute such ownership information. This reduces the overhead of Infinispan, but more importantly "
			+ "improves redundancy as there is no need to replicate the ownership information in case of node failure."
			+ "With this in mind, we can see that virtual nodes are an ideal solution to the distribution of nodes "
			+ "problem, as it allows the location of an entry to be determined algorithmically. Infinispan implements "
			+ "virtual nodes by altering the algorithm for splitting the hash space whenever a node joins or leaves the "
			+ "grid. Rather than allocating a block of the hashspace to the node, it allocates a number of blocks from "
			+ "throughout the hash space. Often it's easier to understand the topology changes that virtual nodes introduce "
			+ "through diagrams.";
	private static final String SHORT_MSG = "test";

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Cache<String, String> c;
		try {
			c = new DefaultCacheManager("xmatter_infinispan.xml")
			// .getCache("replication");
					.getCache("distributedCache");
			String keyProfix = args[0];
			if (args[0].equalsIgnoreCase("a")) {
				long start = System.currentTimeMillis();
				for (int i = 0; i < MAX_ITEMS; i++) {
					String msg = "from cache " + args[0] + LONG_MSG;
					String key = keyProfix + i;
					NotifyingFuture<String> f = c.putAsync(key, msg);
//					c.put(key, msg);
				}
				System.out.println("Take "
						+ (System.currentTimeMillis() - start) + "ms");
			} else {
				for (;;) {
					long start = System.currentTimeMillis();
					int j = 0;
					for (int i = 0; i < MAX_ITEMS; i++) {
						String result = c.get("a" + i);
						if (result != null) {
							j++;
						}
					}
					System.out.println("Read takes "
							+ (System.currentTimeMillis() - start)
							+ "ms. Found number item: " + j);
					Thread.currentThread().sleep(10 * 1000);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
}
